@@ -21,11 +21,16 @@
 
 #include "backend.h"
 
-#include <QtCore/QCoreApplication>
-#include <QtCore/QLatin1Literal>
-#include <QtCore/QtPlugin>
-#include <QtCore/QVariant>
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+#include <QApplication>
+#else
+#include <QGuiApplication>
+#endif
+#include <QIcon>
+#include <QLatin1Literal>
 #include <QMessageBox>
+#include <QtPlugin>
+#include <QVariant>
 
 #include <phonon/GlobalDescriptionContainer>
 #include <phonon/pulsesupport.h>
@@ -102,23 +107,35 @@ Backend::Backend(QObject *parent, const QVariantList &)
             qWarning("WARNING: Setting the user agent for streaming and"
                      " PulseAudio requires you to set QCoreApplication::applicationName()");
         }
-#ifdef __GNUC__
-#warning application name ought to be configurable by the consumer ... new api
-#endif
 #if (LIBVLC_VERSION_INT >= LIBVLC_VERSION(2, 1, 0, 0))
-        if (!qApp->applicationName().isEmpty() && !qApp->applicationVersion().isEmpty()) {
+        PulseSupport::getInstance()->enable(true);
+        const bool pulseActive = PulseSupport::getInstance()->isActive();
+        PulseSupport::getInstance()->enable(false);
+        if (!qApp->applicationName().isEmpty()) {
             const QString id = QString("org.kde.phonon.%1").arg(qApp->applicationName());
             const QString version = qApp->applicationVersion();
-            const QString icon = qApp->applicationName();
+            QString icon;
+#if QT_VERSION >= QT_VERSION_CHECK(4, 7, 0)
+            if (!qApp->windowIcon().isNull()){
+                // Try to get the fromTheme() name of the QIcon.
+                icon = qApp->windowIcon().name();
+            }
+#endif
+            if (icon.isEmpty()) {
+                // If we failed to get a proper icon name, use the appname instead.
+                icon = qApp->applicationName().toLower();
+            }
             libvlc_set_app_id(libvlc,
                               id.toUtf8().constData(),
                               version.toUtf8().constData(),
                               icon.toUtf8().constData());
-        } else if (PulseSupport::getInstance()->isActive()) {
+        } else if (pulseActive) {
             qWarning("WARNING: Setting PulseAudio context information requires you"
-                     " to set QCoreApplication::applicationName() and"
-                     " QCoreApplication::applicationVersion()");
+                     " to set QCoreApplication::applicationName(),"
+                     " QCoreApplication::applicationVersion() and"
+                     " QGuiApplication::windowIcon().");
         }
+
 #endif
     } else {
 #ifdef __GNUC__
@@ -138,9 +155,17 @@ Backend::Backend(QObject *parent, const QVariantList &)
 
     // Initialise PulseAudio support
     PulseSupport *pulse = PulseSupport::getInstance();
-    pulse->enable();
+#if (LIBVLC_VERSION_INT >= LIBVLC_VERSION(2, 2, 0, 0))
+    // VLC 2.2 changed the stream creation order around internally which breaks
+    // the Pulseaudio hijacking. Since VLC upstream doesn't feel like giving us
+    // any more property control we now consider this feature unsupported. As
+    // such whatever properties VLC sets will be what pulse knows about us.
+    pulse->enable(true);
     connect(pulse, SIGNAL(objectDescriptionChanged(ObjectDescriptionType)),
             SIGNAL(objectDescriptionChanged(ObjectDescriptionType)));
+#else
+    pulse->enable(false);
+#endif
 
     m_deviceManager = new DeviceManager(this);
     m_effectManager = new EffectManager(this);
